#!/usr/bin/env python3
"""
Mail1 CLI
by Javantea
Mar 4, 2015

A command-line interface to AltSci Maildir.

Based on the code.
"""
from __future__ import print_function
import os
import sys
import codecs
import re
import time
import tempfile
import bikeim.mail1 as mail1
sanitizeCli = mail1.sanitizeCli
decodeHeaderCli = mail1.decodeHeaderCli
PY2 = mail1.PY2

int_re = re.compile('^[0-9]+$')

def saneInput(prompt):
	"""
	When converting from Python 2 to 3 it will change raw_input to input. 
	If you run input("Username: ") in Python 2, it will execute anything 
	given on the command-line which introduce a privilege escalation bug. 
	This function intends to ensure that does not happen.
	"""
	if sys.hexversion < 0x3000000:
		return raw_input(prompt)
	#end if
	return input(prompt)
#end def saneInput(prompt)

def set_output_encoding(encoding='utf-8'):
	'''When piping to the terminal, python knows the encoding needed, and
	   sets it automatically. But when piping to another program (for example,
	   | less), python can not check the output encoding. In that case, it 
	   is None. What I am doing here is to catch this situation for both 
	   stdout and stderr and force the encoding'''
	current = sys.stdout.encoding
	if current is None:
		sys.stdout = codecs.getwriter(encoding)(sys.stdout)
	current = sys.stderr.encoding
	if current is None:
		sys.stderr = codecs.getwriter(encoding)(sys.stderr)
#end def set_output_encoding(encoding='utf-8')

def drawDir(path = None, currentDir='', page=0, pagesize=122):
	"""
	List page n of e-mails in a directory, pagesize defaults to 122.
	"""
	if path == None:
		#path = '/home/jvoss/Mail'
		path = os.path.join(os.path.expanduser('~'), 'Mail')
	#end if

	# messages on the top right
	output = ''
	index = mail1.getIndex(currentDir, page)
	messages = mail1.readDir(os.path.join(path, currentDir, 'cur'))
	show = messages['files']
	show.sort(reverse=True)
	show = show[page*pagesize:(page+1)*pagesize]
	i = (page * pagesize) + 1
	for message in show:
		fname = os.path.join(path, currentDir, 'cur', message)
		if message not in index:
			fp = open(fname, 'rb')
			msg = mail1.message_from_binary_file(fp, fname)
		else:
			#indexed += 1
			# Pull from the index
			msg = index[message]
		#end if
		subject = msg['subject']
		lsubject = decodeHeaderCli(subject, False, True)
		mfrom = msg['from']
		fenc = decodeHeaderCli(mfrom, False)
		mfrom = sanitizeCli(fenc)
		mto = msg['to']
		tenc = decodeHeaderCli(mto, False)
		mto = sanitizeCli(tenc)
		date = sanitizeCli(msg['date'])
		date_orig = date
		date_format = '%a, %d %b %Y %H:%M:%S %z'
		if PY2:
			date_format = '%a, %d %b %Y %H:%M:%S'
			datel = date.split(u' ')
			date = u' '.join(datel[:5])
			date2 = u' '.join(datel[:4])
		else:
			datel = date.split(' ')
			date = ' '.join(datel[:6])
			if date != '':
				# No sense whatsoever why they would use %Z instead of %z but they do. sendgrid.net, Wells Fargo, didww.com?
				if date[-1] not in '0123456789':
					date_format = '%a, %d %b %Y %H:%M:%S %Z'
				#end if
				date2 = ' '.join(datel[:4])
			else:
				date2 = ''
			#end if
		#end if
		try:
			date = time.strptime(date, date_format)
		except ValueError:
			#print('date', repr(date))
			date = None
		#end try
		if date == None:
			date = date2
			try:
				date = time.strptime(date, '%d %b %Y %H:%M:%S')
			except ValueError:
				print('date', repr(date_orig))
				date = None
			#end try
		#end if
		if date:
			# TODO: Deal with python2 not dealing with the timezone.
			date = time.strftime('%a %b %d %H:%M', date)
		else:
			date = (date_orig.replace(' {0} '.format(time.strftime('%Y')), ' '))[:16].ljust(16, ' ')
		#end if
		spam_level = len(msg.get('X-Spam-Level', ''))
		spf = msg.get('Received-SPF', '')
		if spf[:8].lower() == 'softfail':
			spam_level = max(spam_level, 5)
		elif spf[:4].lower() == 'none':
			spam_level = max(spam_level, 4)
		#end if
		spam_icon = min(spam_level, 10)
		smfrom = mfrom
		win_width = 120
		if len(lsubject) > win_width-(35):
			ssubject = sanitizeCli(lsubject[:win_width-35])
		else:
			ssubject = sanitizeCli(lsubject)
		#end if
		lsubject = sanitizeCli(lsubject)
		if len(fenc) > 18:
			smfrom = sanitizeCli(fenc[:18])
		else:
			smfrom = mfrom
		#end if
		if len(tenc) > 18:
			smto = sanitizeCli(tenc[:18])
		else:
			smto = mto
		#end if
		if 'sent-mail' in currentDir:
			saddr = smto
			#taddr = mto
		else:
			saddr = smfrom
			#taddr = mfrom
		#end if
		#output += fname + ' ' + spam_icon + '\n' + ssubject + '\n' + saddr + '\n' + date + '\n\n'
		output += u'{0:>2d} {1:>2d} {2:<18} {3} {4}\n'.format(spam_icon, i, saddr, date, ssubject)
		i += 1
	#next message
	return output, show
#end def drawDir(path = None, currentDir = '', page = 0)

def drawMsg(filename, add='', use_part=None, index=None):
	html = (add == 'html')
	tryCleanHtml = False
	tryFootnoteHtml = False
	useTextHtml = False
	if add == 'clean_html':
		html = True
		tryCleanHtml = True
	if add == 'footnote_html':
		html = True
		tryFootnoteHtml = True
	elif add == 'text_html':
		html = True
		useTextHtml = True
	#end if
	img = (add == 'img')
	app = (add == 'app')
	gpg = (add == 'gpg')
	
	if index == None:
		index = filename
	else:
		index = str(index)
	#end if

	mime_ext = {'image/jpeg':'jpg', 'image/png':'png', 'image/gif':'gif', 'application/pdf':'pdf',
	'application/zip':'zip', 'text/plain':'txt', 'application/pgp-encrypted':'txt'}
	# TODO: disallow certain extensions
	#disallow_ext = ['exe', 'sh', 'py', 'bin']

	# message body on the bottom right
	#output = '<div class="msgd">\n'
	
	if not os.path.exists(filename):
		return 'File not found. Please try again.'
	#end if
	
	"""
	We don't have write access, so no go. This is easy but silly.
	if currentMessage[-4:] != ':2,S' and currentMessage[-5:] != ':2,RS':
		currentMessage2 = currentMessage + ':2,S'
		# It's been seen, so rename it (this is way weird)
		os.rename(path + '/' + currentDir + '/cur/' + currentMessage, path + '/' + currentDir + '/cur/' + currentMessage2)
		currentMessage = currentMessage2
	#end if
	"""
	
	fp = open(filename, 'rb')
	# TODO: Use this to decide whether we want to read the whole thing.
	#f.seek(0, 2)
	#file_size = f.tell()
	#f.seek(0, 0)
	msg = mail1.message_from_binary_file(fp, filename)
	subject = msg['subject']
	ssubject = decodeHeaderCli(subject)
	mfrom = decodeHeaderCli(msg['from'])
	date = decodeHeaderCli(msg['date'])
	mto = decodeHeaderCli(msg['to'])
	message_id = None
	if 'Message-ID' in msg:
		message_id = decodeHeaderCli(msg['Message-ID'])
	#end if
	mcc = None
	if 'cc' in msg:
		mcc = decodeHeaderCli(msg['cc'])
	#end if
	# msg[] can be email.header.Header instead of str. Silly, but...
	#from_filter = mail1.genFilter(str(msg['from']), 'From')
	#to_filter = mail1.genFilter(str(msg['to']), 'To')

	##output += '<a href="#" onclick="if (window.parent && window.parent.reply) window.parent.reply(document); else alert(\'fixme \' + window.parent.toString());">Reply</a>\n'
	#output += '<a href="mail1mod.py?' + urllib.parse.urlunparse({'dir':currentDir, 'msg': currentMessage, 'junk':'1'}) + '">Spam</a>\n'
	output = 'Subject: ' + ssubject + '\n'
	output += 'From: ' + mfrom + '\n'
	output += 'To: ' + mto + '\n'
	if mcc:
		output += 'Cc: ' + mcc + '\n'
	#end if
	output += 'Date: ' + date + '\n'
	if message_id:
		output += 'Message-ID: ' + message_id + '\n'
	#end if
	#output += '</div>\n'
	#return output
	#output += '<div class="pre">\n'
	output += '\n'
	i = 0
	#output += '<div id="headers" class="msgh2" style="display:none;">\n'
	for (k, v) in msg.items():
		output += sanitizeCli(k + ': ' + str(v)) + '\n'
	#next k,v
	#output += '</div>\n'
	#output += '</div>\n'
	output += '\n'
	#output += '<div class="pre" id="b">\n'
	signed = False
	encrypted = False
	ctypes_available = [part.get_content_type() for part in msg.walk()]
	if add == '' and 'text/plain' not in ctypes_available:
		html = True
		tryCleanHtml = True
	#end if
	for i, part in enumerate(msg.walk()):
		#if part.get_size() > 10000000:
		#	output = ''
		#end if
		ctype = part.get_content_type()
		f_raw = part.get_filename()
		filenameu = ''
		if f_raw == None: f_raw = ''
		if f_raw != '':
			f_decoded = mail1.email.header.decode_header(f_raw)
			if f_decoded[0][1] != None:
				filename_p = sanitizeCli(mail1.guessEncoding(f_decoded[0][0], f_decoded[0][1]))
			else:
				filename_p = sanitizeCli(f_decoded[0][0])
			#end if
			if isinstance(filename_p, str):
				filenameu = filename_p
			else:
				try:
					filenameu = filename_p.decode('utf-8')
				except UnicodeDecodeError:
					filenameu = repr(filename_p)
				#end try
			#end if
		else:
			filename_p = None
		#end if
		#ssize = str(part.get_size())
		ssize = ''
		output += ctype + '\n'
		#fname = part.get_param('name')
		#output += 'dir:' + sanitizeCli(repr(dir(part))) + '<br />\n'
		#output += 'name:' + sanitizeCli(repr(part.get_param('name'))) + '<br />\n'
		#continue
		if ctype in mail1.KNOWN_MULTIPART:
			if ctype == 'multipart/signed':
				signed = True
			#end if
			if ctype == 'multipart/encrypted':
				encrypted = True
			#end if
			pass
			#output += '<div style="border:1px solid black;">\n'
			#for partb in part.walk():
			#	output += sanitizeCli(str(part))
			#	output += '<hr />\n'
			#next partb
			#output += '</div>\n'
		elif ctype.startswith('image/'):
			ext = 'bin'
			if ctype in mime_ext: ext = mime_ext[ctype]
			if filenameu == '': filenameu = ('image{0:03d}.{1}'.format(i, ext))
			if img and (str(i) == use_part):
				return (ctype, part.get_payload(decode=True), None)
			else:
				output += '{0} img {1:d} {2!r}\n'.format(index, i, filenameu)
			#end if
		elif ctype.startswith('application/') or filename_p != None:
			ext = 'bin'
			if encrypted:
				ext = 'gpg'
			#end if
			if ctype in mime_ext: ext = mime_ext[ctype]
			if filenameu == None: filenameu = ('file{0:03d}.{1}'.format(i, ext))
			if app and (str(i) == use_part):
				return (ctype, part.get_payload(decode=True), filename_p)
			else:
				output += '{0} app {1:d} {2!r} {3}\n'.format(index, i, filenameu, ssize)
			#end if
		elif ctype.startswith('audio/'):
			ext = 'bin'
			if ctype in mime_ext: ext = mime_ext[ctype]
			if filenameu == None: filenameu = ('audio{0:03d}.{1}'.format(i, ext))
			if app and (str(i) == use_part):
				return (ctype, part.get_payload(decode=True), filename_p)
			else:
				output += '{0} app {1:d} {2!r}\n'.format(index, i, filenameu)
			#end if
		elif ctype == 'text/html':
			if html == True:
				# Be very careful to ensure this does not get execed.
				pdata = part.get_payload(decode=True)
				ct = part.get('Content-Type')
				charset = None
				if ct and 'charset=' in ct:
					charset = ct[ct.index('charset=')+8:]
				#end if
				pdata = mail1.guessEncoding(pdata, charset)
				# TODO: warning.
				if tryCleanHtml:
					markdown = mail1.subNewline(mail1.anchorToMarkdown(mail1.clean_html(pdata)))
					output += markdown
				elif tryFootnoteHtml:
					markdown = mail1.subNewline(mail1.anchorToFootnote(mail1.clean_html(pdata)))
					output += markdown
				elif useTextHtml:
					output += mail1.subNewline(mail1.text_html(pdata))
				else:
					output += str(pdata)
				#end if
				output += '--------------------------------\n'
			else:
				output += index + ' html\n'
				output += index + ' text_html\n'
				output += index + ' clean_html\n'
				output += index + ' footnote_html\n'
			#end if
		elif ctype == 'text/plain':
			if (app or gpg) and (str(i) == use_part):
				ext = 'txt'
				if filename == None: filename = ('file{0:03d}.{1}'.format(i, ext))
				if gpg:
					# RFC3156 wants unencoded with headers.
					#output = '\n'.join([': '.join(a) for a in part.items()]) + '\n\n'
					#output += part.get_payload(decode=False)
					output = mail1.as_bytes(part)
					# Fix newlines for some weird reason.
					if not b'\r' in output:
						output = output.replace(b'\n', b'\r\n')
					#end if
				else:
					output = part.get_payload(decode=True)
				#end if
				return (ctype, output, filename)
			if html == False:
				#.decode()
				pdata = part.get_payload(decode=True)
				ct = part.get('Content-Type')
				charset = 'utf-8'
				if ct and ('charset=' in ct or 'charset =' in ct):
					if 'charset=' in ct:
						charset = ct[ct.index('charset=')+8:].strip()
					elif 'charset =' in ct:
						charset = ct[ct.index('charset =')+9:].strip()
					#end if
					if ';' in charset: charset = charset[:charset.index(';')]
					if '"' in charset: charset = charset.replace('"','')
				#end if
				pdata = mail1.guessEncoding(pdata, charset)
				output += pdata
				# TODO: warning
				output += '--------------------------------\n'
				if signed:
					output += '{0} gpg {1:d}\n'.format(index, i)
				#end if
				output += '{0} app {1:d}\n'.format(index, i)
			else:
				output += '{0} text\n'.format(index)
			#end if
		else:
			# This is the else for content types. It gets called for:
			#message/delivery-status
			#message/rfc822
			# And other unknown message types.
			pdata = part.get_payload(decode=True)
			ct = part.get('Content-Type')
			charset = None
			if ct and 'charset=' in ct:
				charset = ct[ct.index('charset=')+8:]
			#end if
			# TODO: This is often a binary or something else...
			pdata = mail1.guessEncoding(pdata, charset)
			output += pdata
			# TODO: warning
			output += '--------------------------------\n'
		#end if
		if len(output) > 1000000:
			output += "Debug: It's a big one\n"
		#end if
	#next i, part
	#output += '</div>\n' # pre
	#end if
	#output += '</div>\n'
	
	#output += '</body>'
	#output += '</html>'
	return output
#end def drawMsg(filename, add = '', use_part = None)

def trySaneInput(prompt=''):
	"""
	No way to deal with this pretty I guess I'll just do this.
	FIXME: no correct way to handle these conditions?
	"""
	try:
		command = saneInput(prompt)
	except EOFError:
		return ''
	except KeyboardInterrupt:
		return ''
	#end try
	return command
#end def trySaneInput(prompt='')

def composeHeaders(sfrom='', subject='', to='', cc='', bcc='', header='', config=None):
	"""
	Prompt the user for an e-mail.
	"""
	if not sfrom:
		sfrom = config['Mail1'].get('from', '')
	#end if
	cc = ''
	bcc = ''
	header = ''
	sfrom_asked = False
	if sfrom == '':
		sfrom_asked = True
		sfrom = trySaneInput("From: ")
	#end if
	if to.strip() == '':
		to = None
	#end if
	if to == None:
		to = trySaneInput("To: ")
	#end if
	if not subject.strip():
		subject = trySaneInput("Subject: ")
	#end if
	more = trySaneInput("More headers? (y/n) ")
	if more.lower() in ['y', 'yes', '1', 'ja', 't', 'true']:
		if not sfrom_asked:
			sfrom = trySaneInput("From: ")
		#end if
		cc = trySaneInput("Cc: ")
		bcc = trySaneInput("Bcc: ")
		header = trySaneInput("Headers:\n")
	#end if
	return (sfrom, to, cc, bcc, subject, header)
#end def composeHeaders(to='', subject='', cc='', bcc='', header='', config=None)

def sendWrap(sfrom, to, subject, body, cc='', bcc='', header='', config=None, maildir=''):
	# Copied from mail1glade2.py.
	server = config['Mail1'].get('server')
	server_fingerprint = config['Mail1'].get('server_fingerprint')
	server_dest = None
	if isinstance(server, str):
		server_dest = server.split(':')
		if len(server_dest) != 2:
			server_dest = (server, 25)
		else:
			server_dest[1] = int(server_dest[1])
		#end if
	#end if
	attachments = []
	result = mail1.send(sfrom, to, cc, bcc, subject, body, attach=attachments, 
			headers=header, server=server_dest, maildir=maildir, 
			fingerprint=server_fingerprint)
	# result is a string that should start with Success on success. Not otherwise.
	if not result.startswith('Success'):
		# Tell the user.
		print('Error: Mail Send Failed', result)
		return
	#end if
	#self.sent = True
	return
#end def sendWrap(sfrom, to, subject, body, cc='', bcc='', header='', config=None, maildir='')

def compose(to='', config=None, maildir=''):
	"""
	Prompt the user for an e-mail.
	"""
	sfrom, to, cc, bcc, subject, header = composeHeaders(to=to, config=config)
	# TODO: Sign and encrypt
	print("Body end with ctrl-d")
	body = sys.stdin.read()
	print(sfrom, to, cc, bcc, header, body)
	send = trySaneInput("Send? (y/n) ")
	if send.lower() not in ['y', 'yes', '1', 'ja', 't', 'true']:
		saveDraft = trySaneInput("Save draft? (y/n) ")
		if saveDraft.lower() in ['y', 'yes', '1', 'ja', 't', 'true']:
			# TODO: Save Draft from mail1glade2.py
			return
		#end if
		return
	#end if
	
	sendWrap(sfrom, to, subject, body, cc, bcc, header, config, maildir)
	return
#end def compose(to='', config=None, maildir='')

def reply(filename, msg_id, add='', config=None, maildir=''):
	"""
	Prompt the user for a reply.
	"""
	fp = open(filename, 'rb')
	msg = mail1.message_from_binary_file(fp, filename)
	subject = msg['subject']
	ssubject = decodeHeaderCli(subject)
	mfrom = decodeHeaderCli(msg['from'])
	date = decodeHeaderCli(msg['date'])
	mto = decodeHeaderCli(msg['to'])
	header = ''
	if 'Message-Id' in msg:
		header = ('In-Reply-To: ' + msg['Message-Id'] + "\n" + 'References: ' + msg['Message-Id'] + " " + msg.get('References', ''))
	#end if

	sfrom, to, cc, bcc, subject, header = composeHeaders(to=mfrom, subject='Re: ' + ssubject, header=header, config=config)
	# TODO: Sign and encrypt
	# Leave the file there. We don't care.
	f_file = tempfile.NamedTemporaryFile(delete=False)
	# TODO: get just the body from the drawMsg
	body_data = drawMsg(filename, add)
	body_data = '>' + ('\n>'.join(body_data.split('\n')))
	body_data = ('\n\nOn {0}, {1} wrote:\n'.format(msg['date'], msg['from'])) + body_data
	f_file.write(body_data.encode('utf-8'))
	f_file.flush()
	f_filename = f_file.name
	f_file.close()
	print("Open editor and edit {0}".format(f_filename))
	trySaneInput("Press return when done.")
	#f_file.seek(0)
	f_file = open(f_filename, 'rb')
	body_bin = f_file.read()
	# TODO: Allow the user to pick a different encoding using LANG env var
	body = mail1.guessEncoding(body_bin, 'utf-8')
	print(sfrom, to, cc, bcc, header, body)
	send = trySaneInput("Send? (y/n) ")
	if send.lower() not in ['y', 'yes', '1', 'ja', 't', 'true']:
		saveDraft = trySaneInput("Save draft? (y/n) ")
		if saveDraft.lower() in ['y', 'yes', '1', 'ja', 't', 'true']:
			# TODO: Save Draft from mail1glade2.py
			return
		#end if
		return
	#end if
	
	sendWrap(sfrom, to, subject, body, cc, bcc, header, config, maildir)
	return
#end def reply(msg_id, config=None, maildir='')

def main():
	# Python2 is very dumb sometimes.
	set_output_encoding()
	config = mail1.getConfig()
	if 'Mail1' not in config:
		config['Mail1'] = {'inbox': 'inbox'}
	#end if
	currentDir = config['Mail1'].get('inbox', 'inbox')
	add = ''
	if '--html' in sys.argv:
		sys.argv.remove('--html')
		add = 'html'
	#end if
	if '--clean_html' in sys.argv:
		sys.argv.remove('--clean_html')
		add = 'clean_html'
	#end if
	if '--text_html' in sys.argv:
		sys.argv.remove('--text_html')
		add = 'text_html'
	#end if
	if '-f' in sys.argv:
		sys.argv.pop(sys.argv.index('-f'))
		if len(sys.argv) <= 1:
			exit(1)
		#end if
		filename = sys.argv[1]
		print(drawMsg(filename, add))
		return 0
	#end if
	
	if len(sys.argv) > 1:
		currentDir = sys.argv[1]
	#end if
	Maildir = os.path.join(os.path.expanduser('~'), 'Mail')
	
	# custom pager like nail/mailx
	try:
		import readline
		readline.parse_and_bind('tab: complete')
	except ImportError as ie:
		print("Warning: readline isn't available no input helpers.")
	#end try
	win_height = 40
	i = 0
	running = True
	command = 'ls'
	show = []
	show_i = None
	msg_i = None
	add_temp = add
	# Command line loop
	while running:
		if command == 'ls' or command == '':
			x, show = drawDir(Maildir, currentDir, i, win_height)
			if x:
				print(x)
			#end if
			i += 1
		#end if
		try:
			command = saneInput('# ')
		except EOFError:
			running = False
			break
		except KeyboardInterrupt:
			running = False
			break
		#end try
		if command == 'q' or command == 'quit' or command == 'exit':
			# Quit
			print("Goodbye")
			running = False
			break
		elif int_re.match(command[:1]):
			# Read a message by id
			add_temp = add
			use_part = None
			if ' ' in command:
				command, add_temp = command.split(' ', 1)
				add_temp = add_temp.strip()
				if ' ' in add_temp:
					add_temp, use_part = add_temp.split(' ', 1)
					add_temp = add_temp.strip()
				#end if
			msg_i = int(command)
			show_i = msg_i - (win_height * max(0, i - 1)) - 1
			if show_i < 0 or show_i >= len(show):
				# 0 = msg_i - (win_height * (i - 1)) - 1
				# win_height - 1 = msg_i - (win_height * (i - 1)) - 1
				print("Outside valid range", show_i, i, msg_i, 'vs {0}-{1}'.format((win_height * max(0, i - 1)) + 1, win_height * i))
				continue
			#end if
			fn = os.path.join(Maildir, currentDir, 'cur', show[show_i])
			msg_data = drawMsg(fn, add_temp, use_part, msg_i)
			if isinstance(msg_data, tuple):
				# save it to a file
				ctype, data, filename = msg_data
				out_filename = saneInput('Filename: ')
				if out_filename:
					open(out_filename, 'wb').write(data)
				#end if
			else:
				print(msg_data)
			#end if
		elif command == 'cd' or command.startswith('cd '):
			# Change directory
			# Assume the user isn't actually trying to directory traversal.
			while ('..' + os.sep) in command:
				command = command.replace('..' + os.sep, '')
			#loop
			currentDir = command[3:]
			i = 0
			msg_i = None
			x, show = drawDir(Maildir, currentDir, i, win_height)
		elif command == 'c' or command.startswith('c ') or command.startswith('compose'):
			# Compose a message
			pos = 1
			if command.startswith('c '):
				pos = 2
			elif command.startswith('compose'):
				pos = 8
			#end if
			compose(command[pos:], config, Maildir)
		elif command == 'r' or command.startswith('r ') or command.startswith('reply'):
			# Reply to a message
			pos = 1
			if command.startswith('r '):
				pos = 2
			elif command.startswith('reply'):
				pos = 6
			#end if
			if int_re.match(command[pos:].strip()[:1]):
				try:
					msg_i = int(command[pos:].strip())
				except ValueError as ve:
					print("Usage: r [msg_id]")
					print("msg_id must be a number")
				#end try
			#end if
			if msg_i == None:
				# They haven't read a message.
				print("You can only reply to a message after reading it or by giving the msg_id on the command line.")
				continue
			#end if
			#print("i", i, "msg_i", msg_i, "", )
			show_i = msg_i - (win_height * max(0, i - 1)) - 1
			if show_i < 0 or show_i >= len(show):
				# 0 = msg_i - (win_height * (i - 1)) - 1
				# win_height - 1 = msg_i - (win_height * (i - 1)) - 1
				print("Outside valid range", show_i, i, msg_i, 'vs {0}-{1}'.format((win_height * max(0, i - 1)) + 1, win_height * i))
				continue
			#end if
			fn = os.path.join(Maildir, currentDir, 'cur', show[show_i])
			reply(fn, msg_i, add_temp, config, Maildir)
		elif command.startswith('h') or command.startswith('?') or command.startswith('help'):
			# Help
			print("ls     | list messages")
			print("cd dir | change directory")
			print("quit   | quit")
			print("1      | print the first message")
			print("1 html | print the first message's html part")
			print("       | list messages and increment page number")
			print("c      | compose a message")
			print("r      | reply to a message")
		#end if
	#loop
	print()
#end def main()

if __name__ == '__main__':
	main()
#end if
